package be.iminds.ilabt.jfed.rspec;

import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonProperty;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.*;

/**
 * config for AdvertisementBasedRspecGenerator
 */
public class AdvertisementBasedRspecGeneratorConfig {
    @Nonnull public final List<NodeInfo> nodes;

    @JsonCreator
    public AdvertisementBasedRspecGeneratorConfig(
            @Nonnull @JsonProperty("nodes") List<NodeInfo> nodes) {
        this.nodes = nodes;
    }

    @Nonnull
    public List<NodeInfo> getNodes() {
        return nodes;
    }

    public static class NodeInfo {
        public final int count;
        @Nonnull public final String clientIdBase;
        @Nullable public final String ansibleGroup;
        @Nullable public final String diskImage;
        @Nullable public final List<String> componentId;
        @Nullable public final String sliverType;
        @Nullable public final List<String> hardwareTypeFilter;
        @Nullable public final Boolean availableFilter;
        @Nullable public final Boolean exclusive;

        public NodeInfo(          int count,
                        @Nullable String clientIdBase,
                        @Nullable String ansibleGroup,
                        @Nullable String diskImage,
                        @Nullable List<String> componentId,
                        @Nullable String sliverType,
                        @Nullable List<String> hardwareTypeFilter,
                        @Nullable Boolean availableFilter,
                        @Nullable Boolean exclusive) {
            this.count = count;
            this.clientIdBase = clientIdBase == null ? "node" : clientIdBase;
            this.ansibleGroup = ansibleGroup;
            this.diskImage = diskImage;
            this.componentId = componentId;
            this.sliverType = sliverType;
            this.hardwareTypeFilter = hardwareTypeFilter;
            this.availableFilter = availableFilter;
            this.exclusive = exclusive;
        }

        @JsonCreator
        public static NodeInfo createNodeInfoFromJsonObject(@Nonnull Map<String,Object> props) {
            //TODO: Check if types are correct etc
            int count = (int) props.get("count");
            List<String> componentId = null;
            Object componentIdObj = props.get("componentId");
            if (componentIdObj != null) {
                if (componentIdObj instanceof Collection) {
                    componentId = new ArrayList((Collection) componentIdObj);
                }
                if (componentIdObj instanceof String) {
                    componentId = Collections.singletonList((String) componentIdObj);
                }
                assert componentId != null;
            }
            List<String> hardwareTypeFilter = null;
            Object hardwareTypeFilterObj = props.get("hardwareTypeFilter");
            if (hardwareTypeFilterObj != null) {
                if (hardwareTypeFilterObj instanceof Collection) {
                    hardwareTypeFilter = new ArrayList((Collection) hardwareTypeFilterObj);
                }
                if (hardwareTypeFilterObj instanceof String) {
                    hardwareTypeFilter = Collections.singletonList((String) hardwareTypeFilterObj);
                }
                assert hardwareTypeFilter != null;
            }
            return new NodeInfo(count,
                    (String) props.get("clientIdBase"),
                    (String) props.get("ansibleGroup"),
                    (String) props.get("diskImage"),
                    componentId,
                    (String) props.get("sliverType"),
                    hardwareTypeFilter,
                    (Boolean) props.get("availableFilter"),
                    (Boolean) props.get("exclusive"));
        }

        public int getCount() {
            return count;
        }

        @Nonnull
        public String getClientIdBase() {
            return clientIdBase;
        }

        @Nullable
        public String getAnsibleGroup() {
            return ansibleGroup;
        }

        @Nullable
        public String getDiskImage() {
            return diskImage;
        }

        @Nullable
        public List<String> getComponentId() {
            return componentId;
        }

        @Nullable
        public String getSliverType() {
            return sliverType;
        }

        @Nullable
        public List<String> getHardwareTypeFilter() {
            return hardwareTypeFilter;
        }

        @Nullable
        public Boolean getAvailableFilter() {
            return availableFilter;
        }

        @Nullable
        public Boolean getExclusive() {
            return exclusive;
        }
    }
}
